%%This function solves the dynamic programming problem via Bellman
%iteration.
%
%Rather than the value function, we return a conditional choice
%probability function, i.e., a function that returns the probability of
%engine replacement at a given state. Since the multi-index (iid)
%states and the only enter through scalar linear combinations the CCP 
%function take these combinations as arguments rather than the
%vector-valued multi-index (iid) states themselves.
%
%Ben Deaner 2020 (bdeaner@mit.edu or bendeaner@gmail.com)

function [CCP_function]=DPsolve_cont3(params, s_grid, z_grid, tn)
    %The arguments:
    %params - structure containing parameter values
    %s_grid - mileage grid-points at which to iterate the Bellman operator
    %z_grid - states that effect the mileage transitions enter through a scalar
          %linear combination. this variable contains the grid of points of 
          %this linear combination at which to iterate the Bellman operator
    %tn - number of Monte Carlo draws used for the expectation over
      %continuation payoffs in the LHS of the Bellman equation.

    z_dimension=size(params.multi_index,2); %number of iid state variables that effect transitions

    %grid sizes:
    size_s=size(s_grid,2);
    size_z=size(z_grid,2);
 
    %we need to interpolate between the grid points at which we iterate the
    %Bellman operator, to do this we use MATLAB's meshgrid and interpolation
    %functions. If the heterogeneity is iid it does not enter the expected
    %value function so the value function effectively has two dimensions rather
    %than three.
    %for some inexplicaple reason MATLAB generates meshgrids with the first two
    %dimensions flipped, which is why the indexing might look weird.
    [S_grid,Z_grid]=meshgrid(s_grid,z_grid);

    EV1 = zeros(size_z,size_s); %initial guess of the expected value function
    step_change=1;

    %we need to draw tn next period mileages for the monte carlo expectation in
    %the Bellman RHS.
    %uncomment the next line and comment out the one after it if you want
    %replacement states to depend on the multi-index
    %replacement_transitions = 1+normrnd(repmat(params.state_transition(1)+z_grid',size(s_grid,2),1,size(w_grid,2),tn),params.state_transition(2)).^2; %Mileage transitions (non-renewal) %Mileage transitions (non-renewal)
    replacement_transitions = 1+normrnd(repmat(params.state_transition(1),size_z,size_s,tn),params.state_transition(2)).^2; %Mileage transitions (renewal)
    maintenance_transitions = repmat(s_grid,size_z,1,tn)+normrnd(repmat(params.state_transition(1)+z_grid',1,size_s,tn),params.state_transition(2)).^2; %Mileage transitions (non-renewal)

    % the transition states variables are each iid chi-squared or bernoulli and
    % are arranged chi^2, Bernoulli, chi^2, Bernoulli...    
    Zchi=chi2rnd(1,1,ceil(z_dimension/2),tn);
    Zdisc=1*binornd(1,0.5,1,floor(z_dimension/2),tn);
    Ztemp=cat(1,Zchi(:,1:end-mod(z_dimension,2),:),Zdisc);
    Ztemp=reshape(Ztemp,[2*floor(z_dimension/2),tn]);
    z_draws_temp=cat(1,Ztemp,reshape(Zchi(end+1-mod(z_dimension,2):end,1,:),[1,tn]));

    %multi-index 9iid) states and utility heterogeneity may be multi-dimensional but
    %only matter through the linear combinations below.
    z_draws=repmat(permute(params.multi_index*z_draws_temp,[1,3,2]),size_z,size_s,1);

    %now iterate the Bellman until there isonly a tiny change between iterations
    while step_change>0.0000001  %while tolerance level not yet reached


            replacement_EV=interp2(S_grid,Z_grid,EV1,replacement_transitions,z_draws,'linear',-1000000);
            maintenance_EV=interp2(S_grid,Z_grid,EV1,maintenance_transitions,z_draws,'linear',-1000000);
     
            EV=log(exp(params.maintenance_factor*sqrt(S_grid)+params.beta*mean(maintenance_EV,3))...
               +exp(params.replacement_cost+params.beta*mean(replacement_EV,3)));

        
        step_change=abs(EV-EV1);
        step_change=max(step_change(:));
        EV1=EV;
  
    end
    
    %%Evaluate choice probabilities and define a function CCP_function that 
    %returns them. Note that the function must interpolate between choice
    %probabilities at grid points 
    
    replacement_EV=interp2(S_grid,Z_grid,EV1,replacement_transitions,z_draws,'spline',-1000000);
    maintenance_EV=interp2(S_grid,Z_grid,EV1,maintenance_transitions,z_draws,'spline',-1000000);            

    [S_grid,Z_grid]=meshgrid(s_grid,z_grid); 
    
    %on a grid of states, evaluate choice probabilties implied by the logit
    %choice probability formula and the expected value function evaluated 
    %above:
    CCP_grid=1./(1+exp(params.maintenance_factor*sqrt(S_grid)+params.beta*mean(maintenance_EV,3)...
             -(params.replacement_cost+params.beta*mean(replacement_EV,3))));
    
    CCP_function=@(s,z) interp2(S_grid,Z_grid,CCP_grid,s,z,'linear',1);
end