%%This function generates choice and state data
%The choices are drawn from a precalculated conditional choice probabilty
%function which is the argument CCP_function.
%
%The columns of the returned data set contain respectively: choice data,
%mileage, the states that effect only transitions, the utility
%heterogeneity and then the values of each of these variables for the next
%period (it is convenient to have the next period values adjacent for
%estimation).
%
%The function also returns duration data in case we want to check that out.
%Ben Deaner 2020 (bdeaner@mit.edu or bendeaner@gmail.com)

function [data,durations]=MCdata3(CCP_function, params, sample_size, periods)
    %The arguments:
    %CCP function - function that returns probability of replacement at each a
                %state. Pre-calculated using DP_solvecont2
    %params - structure containing parameter values
    %sample_size - number of 'buses' for which to draw data
    %periods - number of periods for which each bus is observed

    burn_in=10; %burn-in period so that the mileage for each bus does not start at 1
    z_dimension=size(params.multi_index,2); %number of iid state variables that effect transitions
    data=zeros(sample_size,periods,4+2*z_dimension); %where we will store the data

    % the transition states variables are each iid chi-squared or bernoulli and
    % are arranged chi^2, Bernoulli, chi^2, Bernoulli...
    Zchi=chi2rnd(1,1,ceil(z_dimension/2),sample_size*(periods+burn_in+1));
    Zdisc=1*binornd(1,0.5,1,floor(z_dimension/2),sample_size*(periods+burn_in+1));
    Ztemp=cat(1,Zchi(:,1:end-mod(z_dimension,2),:),Zdisc);
    Ztemp=reshape(Ztemp,[2*floor(z_dimension/2),sample_size*(periods+burn_in+1)]);
    % The draws of the transition effecting states are below. note we need 
    %extra draws  for the burn-in period and the priods+1-th draw     
    Z_draws=(cat(1,Ztemp,reshape(Zchi(end+1-mod(z_dimension,2):end,1,:),[1,sample_size*(periods+burn_in+1)])))';

    %Transition states and utility heterogeneity may be multi-dimensional but
    %only matter throw the linear combinations below.
     
    z_draws=Z_draws*params.multi_index';

    %The blocks below describe the indeces in which we save each of the 
    %variables. Variables appended with an 'n' refer to the next period values    
    z_block=3:3+z_dimension-1;
    choicen_block=3+z_dimension;
    sn_block=3+z_dimension+1;
    zn_block=3+z_dimension+2:3+(2*z_dimension)+1;

    durations=[];
    
    %now let's draw the data
    for i=1:sample_size
        s=1;
        last_replacement=1;
        for t=1:periods+burn_in %add burn-in periods
            index=(i-1)*(periods+burn_in+1)+t;
  
            
        %choices drawn using precalculated CCP function
            choice = binornd(1,CCP_function(s,z_draws(index)));
                
            if t>burn_in
                %save this period's data
                data(i,t-burn_in,1)=choice;
                data(i,t-burn_in,2) = s;
                data(i,t-burn_in,z_block) = Z_draws(index,:);
                
                data(i,t-burn_in,zn_block) = Z_draws(index+1,:);
                if t>burn_in+1
                    data(i,t-burn_in-1,choicen_block) = choice;
                end
                if  choice
                    durations=[durations;t-max(last_replacement,burn_in)];
                    last_replacement=t;
                end
            end
            
            if(choice==1)
                s=1; 
            end
            %simulate the next period mileage:
            s = s+normrnd(params.state_transition(1)+z_draws(index),params.state_transition(2)).^2;
            if t>burn_in            
                data(i,t-burn_in,sn_block) = s;
            end
            
        end
        choice = binornd(1,CCP_function(s,z_draws(index)));
        data(i,t-burn_in,choicen_block) = choice;        
    end

end