%%The function estimate3 is used to estimate utility function parameters
%and standard errors for the bus engine replacement Monte Carlo simulation.
%
% The function returns two matrices. The first 'estimates' is a 2-by-4 matrix whose columns correspond to estimates
%of the two parameters. The first column has no bias correction, the final
%column is fully bias corrected. The 2nd and 3rd columns are partially
%adjusted using only \phi_2 and using \phi_2 and \phi_3 respectively.
%'standard_errors'is a 2-by-4 matrix whose entries are estimated standard 
%errors for the corresponding elements of 'estimates'.
%
%The function fullinteracts at the bottom of the file is used to generate
%the basis functions when 'powers' is greater than or equal to 2.
%
%Ben Deaner 2020 (bdeaner@mit.edu or bendeaner@gmail.com)

function [estimates,standard_errors]=estimate3(Data,I,statedim,delta,powers)

    %inputs:
    %Data-dataset
    %I-indexes the sub-samples for sample splitting
    %statedim-the number of multi-index or iid state variables (i.e.,
    %states other than the mileage)
    %delta-discount factor
    %powers-determines which basis functions to use. 1 corresponds to
    %linear terms only, 1.5 to linear terms and squares, >=2 corresponds to
    %all terms of order less that 'powers', for example powers=2 implies
    %linear terms, squares, and interactions of all pairs of variables
    
    
    %define functions under the names given in the paper:
    Lambda=@(u) 1./(1+(exp(-u)));
    dLambda=@(u) exp(-u).*(Lambda(u).^2);
    lambda=@(u) dLambda(u)./(Lambda(u).*(1-Lambda(u)));
    H=@(p) -log(1-p);
    H_p=@(p) 1./(1-p);

    rule_of_thumb=0; %set this to 1 to use rule-of-thumb penalty parameters
    %instead of cross-validation
    
    %rename columns in Data to match up with the paper 
    y = 1-Data(:,1); %Y_2 in the document (y(i)=1 if continue with engine and y(i)=0 if replace)
    x = Data(:,2:1+statedim);
    y_n = 1-Data(:,2+statedim);
    x_n = Data(:,3+statedim:end);
    N = size(Data,1);
    
    %generate basis functions corresponding to the choice given in the input  'powers'
    if powers==1
        X=x;
        X_n=x_n;
    end
    if powers==1.5
        X=[x,x.^2];
        X_n=[x_n,x_n.^2];    
    end
    if powers>=2
    [~,powers]=fullinteracts(x,powers);
    X=x2fx(x,powers);
    X_n=x2fx(x_n,powers);
    end

    rule_of_thumb_penalty=2*0.5*norminv(1-0.1/(2*size(X,2)));
    
    %we will save nonparametric regression functions here:
    g_1_hat=nan(N,size(I,2),size(I,2));
    Hg_1_hat=nan(N-(N/size(I,2)),size(I,2));
    g_1_tilde=nan(N,size(I,2));
    g_1_sub=nan(N,1);
    g_3_hat=nan(N,1);
    

    for i=1:size(I,2)        
            %estimate gamma_1 and H() of gamma_1 with two-way sample splitting
            for j=1:size(I,2)
                comp=logical(ones(N,1)-I(:,i)-I(:,j));
                if rule_of_thumb
                    [params,fitinfo]=lasso(X(comp,:),y(comp),'Lambda',rule_of_thumb_penalty/sum(comp,1),'CV',2);
                else
                    [params,fitinfo]=lassoglm(X(comp,:),y(comp),'binomial','CV',2);
                end
                idx= fitinfo.IndexMinDeviance;
                coef=[fitinfo.Intercept(idx);params(:,idx)];
                g_1_hat(:,i,j)=glmval(coef,X_n,'logit');
                g_1_hat(:,i,j)=min(max(g_1_hat(:,i,j),0.0001),0.9999);
                Hg_1_hat(I(:,j),i)=H(g_1_hat(I(:,j),i,j));
            end
            comp=logical(ones(N,1)-I(:,i));

            %estimate gamma_1 and H() of gamma_1 with one-way sample splitting            
            if rule_of_thumb
                [params,fitinfo]=lasso(X(comp,:),y(comp),'Lambda',rule_of_thumb_penalty/sum(comp,1),'CV',2);
            else
                [params,fitinfo]=lassoglm(X(comp,:),y(comp),'binomial','CV',2);
            end

            idx= fitinfo.IndexMinDeviance;
            coef=[fitinfo.Intercept(idx);params(:,idx)];
            g_1_tilde(:,i)=glmval(coef,X_n,'logit');
            
            g_1_tilde(:,i)=min(max(g_1_tilde(:,i),0.0001),0.9999);
            g_1_sub(I(:,i),:)=g_1_tilde(I(:,i),i);
            %esimate gamma_3 with one-way sample splitting
            g_3_hat(I(:,i),:)=mean(H(g_1_tilde(comp,i)).*(1-y(comp)),1)/mean(1-y(comp,1),1);        
    end

    % estimate gamma_2 with sample_splitting
    g_2_hat=nan(N,1);
    for i=1:size(I,2)
            comp=logical(ones(N,1)-I(:,i));
            if rule_of_thumb
                [params,fitinfo]=lasso(y(comp).*X(comp,:),y(comp).*Hg_1_hat(comp,i),'Lambda',rule_of_thumb_penalty/sum(comp,1),'CV',2);
            else
                [params,fitinfo]=lasso(y(comp).*X(comp,:),y(comp).*Hg_1_hat(comp,i),'CV',2);
            end
            idx=fitinfo.IndexMinMSE;
            g_2_hat(I(:,i),:)=[ones(N/size(I,2),1),X(I(:,i),:)]*[fitinfo.Intercept(idx);params(:,idx)];         
    end
 
    %estimate the parameters in a first-pass without any bias correction
    %with sample splitting
    
    Vdiff=delta*(g_2_hat-g_3_hat);
    d=[sqrt(x(:,1)),-ones(N,1)];

    estimate=nan(2,size(I,2));
    v_hat=nan(N,1);
 
    for i=1:size(I,2)
        comp=logical(ones(N,1)-I(:,i));
        mom=@(theta) d(comp,:).*repmat(lambda((d(comp,:)*theta)+Vdiff(comp,:)).*(y(comp,:)-Lambda((d(comp,:)*theta)+Vdiff(comp,:))),[1,2]);
        G=@(theta) sum(mean(mom(theta),1).^2,2);
        estimate(:,i)=fminunc(G,[0;0]);
        v_hat(I(:,i),:)=(d(I(:,i),:)*estimate(:,i))+Vdiff(I(:,i),:);
    end
 
    %now we have the unadjusted estimates on the complement of each sub
    %sample we can use these to calculate the adjustment terms phi_1, phi_2
    %and phi_3
    
    %these objects will be used later on, they correspond to variables in
    %the paper with similar names
    Lambda_hat=Lambda(v_hat);
    dLambda_hat=dLambda(v_hat);
    lambda_hat=lambda(v_hat);
    lamb_part=(lambda_hat.*dLambda_hat.*y)./Lambda_hat;
    
    %evalaute the adjustment term phi_2
    alpha2=-delta*d.*lamb_part;
    phi2=alpha2.*(H(g_1_sub)-g_2_hat);

    A=nan(N,2);
    P1=nan(N,1);
    
    
    %evalaute the adjustment term phi_3 using Lasso    
     for i=1:size(I,2)
            comp=logical(ones(N,1)-I(:,i));
            if rule_of_thumb
                [params,fitinfo]=lasso(X_n(comp,:),1-y(comp),'Lambda',rule_of_thumb_penalty/sum(comp,1),'CV',2);
            else
               [params,fitinfo]=lasso(X_n(comp,:),1-y(comp),'CV',2);
            end  
            idx=fitinfo.IndexMinMSE;
            b_rho3(I(:,i),:)=[ones(sum(I(:,i),1),1),X_n(I(:,i),:)]*[fitinfo.Intercept(idx);params(:,idx)];         
            A(I(:,i),:)=repmat(delta*mean(d(comp,:).*lambda_hat(comp,:).*dLambda_hat(comp,:),1),sum(I(:,i),1),1);
            P1(I(:,i),:)=mean(1-y(comp),1);
     end
     
    alpha3=A.*b_rho3.*H_p(g_1_sub)./P1;
    phi3=(alpha3.*(y_n-g_1_sub))+((A.*(1-y)./P1).*(H(g_1_sub)-g_3_hat));

    %evalaute the adjustment term phi_1 using Lasso
    zeta=nan(N,2);
    
    for j=1:2
        for i=1:size(I,2)
            comp=logical(ones(N,1)-I(:,i));
            if rule_of_thumb
                [params,fitinfo]=lasso(X_n(comp,:),-delta*d(comp,j).*lamb_part(comp),'Lambda',rule_of_thumb_penalty/sum(comp,1),'CV',2);
            else
                     [params,fitinfo]=lasso(X_n(comp,:),-delta*d(comp,j).*lamb_part(comp),'CV',2);
            end
                idx=fitinfo.IndexMinMSE;
            zeta(I(:,i),j)=[ones(sum(I(:,i),1),1),X_n(I(:,i),:)]*[fitinfo.Intercept(idx);params(:,idx)];         
        end   
    end
    
    alpha1=zeta.*H_p(g_1_sub);    
    phi1=alpha1.*(y_n-g_1_sub);
    
    %now we have evaluated all of the adjustment terms we estimate
    %coefficients using various choices of adjustments
    
    %uncorrected estimates:
    mom=@(theta) d.*repmat(lambda((d*theta)+Vdiff).*(y-Lambda((d*theta)+Vdiff)),[1,2]);
    G=@(theta) sum(mean(mom(theta),1).^2,2);
    est1=fminunc(G,[0;0]);

    %estimates with correction term phi_2
    mom_2=@(theta) mom(theta)+phi2;
    G=@(theta) sum(mean(mom_2(theta),1).^2,2);
    est2=fminunc(G,[0;0]);
    %estimates with correction terms phi_2 and phi_3
    mom_23=@(theta) mom(theta)+phi2+phi3;
    G=@(theta) sum(mean(mom_23(theta),1).^2,2);
    est3=fminunc(G,[0;0]);
    %fully corrected estimates i.e., with phi_1, phi_2, and phi_3
    mom_123=@(theta) mom(theta)+phi1+phi2+phi3;
    G=@(theta) sum(mean(mom_123(theta),1).^2,2);
    est4=fminunc(G,[0;0]);
    
    %calculate standard errors using the standard formula using the fully
    %adjusted moments and the derivatives of the moments calculated using
    %first differences
    diff=0.00001; %first different for numerical derivative calculation
    
    dG=nan(2,2);
    mM=@(theta) mean(mom(theta),1);
    dG(1,:)=(mM(est1+[diff;0])-mM(est1))./diff;
    dG(2,:)=(mM(est1+[0;diff])-mM(est1))./diff;
    Om=cov(mom(est1));
    covar=(dG\Om/(dG'))./N;
    sd1=sqrt(diag(covar));
    
    dG=nan(2,2);
    mM=@(theta) mean(mom_2(theta),1);
    dG(1,:)=(mM(est2+[diff;0])-mM(est2))./diff;
    dG(2,:)=(mM(est2+[0;diff])-mM(est2))./diff;
    Om=cov(mom_123(est2));
    covar=(dG\Om/(dG'))./N;
    sd2=sqrt(diag(covar));
    
    dG=nan(2,2);
    mM=@(theta) mean(mom_23(theta),1);
    dG(1,:)=(mM(est3+[diff;0])-mM(est3))./diff;
    dG(2,:)=(mM(est3+[0;diff])-mM(est3))./diff;
    Om=cov(mom_123(est3));
    covar=(dG\Om/(dG'))./N;
    sd3=sqrt(diag(covar));
    
    dG=nan(2,2);
    mM=@(theta) mean(mom_123(theta),1);
    dG(1,:)=(mM(est4+[diff;0])-mM(est4))./diff;
    dG(2,:)=(mM(est4+[0;diff])-mM(est4))./diff;
    Om=cov(mom_123(est4));
    covar=(dG\Om/(dG'))./N;
    sd4=sqrt(diag(covar));
    
    %we are done, return the results
    estimates=[est1,est2,est3,est4];
    standard_errors=[sd1,sd2,sd3,sd4];
    
  
   

end

%the following function is used in generating the basis functions when 
%'powers' is set to be >=2 
function [interacts,inters] = fullinteracts(x,maxpower)

    k=size(x,2);
    Ik=[zeros(1,k);eye(k)];
    tmp=Ik;

    for i=1:maxpower-1
        tmp=repmat(Ik,[size(tmp,1),1])+repelem(tmp,k+1,1);
    end

    inters=unique(tmp,'rows');
    allinters=x2fx(x,inters);

    %let's remove all constant terms
    varx=var(allinters,0,1);
    interacts=allinters(:,logical(varx>0));
    inters=inters(logical(varx>0),:);

    %let's remove all duplicate columns
    [a,b]=unique(interacts','rows');

    interacts=a';
    inters=inters(b,:);

end