%%This code generates simulated datasets for use in the engine replacement
%Monte Carlo simulation. Given the parameters the function DPsolve_cont3.m
%solves the dynamic programming problem faced by the agent. The function
%MCdata3.m then generates datasets using the solution to the dynamic
%programming problem. The data has both a time dimension with size set by 
%the variable 'periods' and an individual dimension set according to the 
%variable 'sample_size'. The number of datasets generated is determined by
%the variable 'draws'.
%
%The simulated datasets are saved to the path in 'save_path'. The path is
%filename is generated automatically based on the sample size.
%The saved output is a four-dimensional array. The rows of 
%the final output correspond to different individuals, the third dimension 
%to differnt time periods and the fourth dimension to different datasets. 
%The columns of the returned data set contain respectively: choice data, 
%mileage, the states that effect only transitions, the utility
%heterogeneity and then the values of each of these variables for the next
%period (it is convenient to have the next period values adjacent for
%estimation).
%
%Ben Deaner 2020 (bdeaner@mit.edu or bendeaner@gmail.com)

clear
rng(2); %set the seed

%% Set paramter values, sample size and number of datasets to generate

sample_size=300; %number of individual engines in each Monte Carlo sample


constant_heterogeneity=2; %is the heterogeneity in the utility function constant or iid?
       
params.maintenance_factor = -0.5;  %per-period maintenance cost coeff
params.replacement_cost = -1;  %replacement cost
params.state_transition = [0.2,1];  %the constants in the mileage transition density
params.beta = 0.9; %discount factor
number_multi_index_states=5; %number of the states that effect the mileage transitions through a multi-index
params.multi_index=0.1*linspace(1,number_multi_index_states,number_multi_index_states).^(-2); %multiindex coefficients for the 

draws=500; %number of Monte Carlo draws for which to generate data
periods=10; %number of individual engines in each Monte Carlo sample
 
save_path=strcat('MCdata_t=',num2str(periods),'_n=',num2str(sample_size),'.mat');

s_grid = linspace(0,100,100);   %mileage grid points on which to iterate the Bellman equation    
z_grid = linspace(0,10,100);   %multi-index grid points on which to iterate the Bellman equation

tn = 10000;  %number of draws used to form monte carlo expectation in DP solution and data generation
        
%% solve the dynamic programming problem by Bellman iteration

fprintf('Solving the dynamic programming problem... \n');
[CCP_function] = DPsolve_cont3(params, s_grid, z_grid, tn);
fprintf('Solved! \n');       

%% generate simulated data

data=nan(sample_size,periods,4+2*(number_multi_index_states),draws);

parfor b=1:draws
        datatmp = MCdata3(CCP_function, params, sample_size, periods);         
        data(:,:,:,b)=datatmp;
end
    
save(save_path,'data','-v7.3');
    
%{
    %%generate histograms for the mileage
    figure
        dt=data(:,:,2,:);
        hist(dt(:));
        h=title('Mileage histogram');
        set(h);
    figure
        dt=data(:,1,2,:);
        hist(dt(:));
        h=title('Mileage t=1');
        set(h);
    figure
        dt=data(:,2,2,:);
        hist(dt(:));
        h=title('Mileage t=2');
        set(h);
    figure
        dt=data(:,end,2,:);
        hist(dt(:));
        h=title('Mileage final t');
        set(h);
        %}